searchNodes=[{"doc":"Perpetual is a simple abstraction around repeatedly iterating state.It is similar to Elixir's Agent module in that it can share or store state that must be accessed from different processes or by the same process at different points in time, and in additiion to that, Perpetual lets you define a function for repeatedly updating the stored state for as long as the process is kept running.The Perpetual module provides a basic server implementation that defines an update function to be repeatedly applied, and allows current state to be retrieved and updated manually via a simple API.ExamplesFor example, the following server implements an infinite counter:defmodule Counter do use Perpetual def start_link(initial_count) do args = [init_fun: fn -&gt; initial_count end, next_fun: &amp;(&amp;1 + 1)] Perpetual.start_link(args, name: __MODULE__) end def get_count do Perpetual.get(__MODULE__, &amp;(&amp;1)) end def stop do Perpetual.stop(__MODULE__) end endUsage would be:Counter.start_link(0) #=&gt; {:ok, #PID&lt;0.123.0&gt;} current_value = Counter.get_count later_value = Counter.get_count Counter.stop #=&gt; :okIn the counter example above, the server will keep counting until the process is stopped. Each call to Counter.value() would retrieve the current count.Perpetual provides a segregation between the client and server APIs (similar to GenServers). In particular, the functions passed as arguments to the calls to Perpetual functions are invoked inside the server. This distinction is important because you may want to avoid expensive operations inside the server for calls to get the current value, as they will effectively block the server until the request is fulfilled. However, it is reasonable to do expensive work as necessary in the next_fun function as that function's work is the whole point of iterating perpetually--just be aware of the blocking effect is has on other messages. Perpetual is designed to be long-running and for clients to request the current state only periodically.How to superviseA Perpetual server is most commonly started under a supervision tree. When we invoke use Perpetual, it automatically defines a child_spec/1 function that allows us to start the server directly under a supervisor. To start the Counter example under a supervisor with an initial counter of 0, one may do:children = [ {Counter, 0} ] Supervisor.start_link(children, strategy: :one_for_all)While one could also simply pass the Counter as a child to the supervisor, such as:children = [ Counter # Same as {Counter, []} ] Supervisor.start_link(children, strategy: :one_for_all)The definition above wouldn't work for this particular example, as it would attempt to start the counter with an initial value of an empty list. However, this may be a viable option in your own servers. A common approach is to use a keyword list, as that would allow setting the initial value and giving a name to the counter process, for example:def start_link(opts \\ []) do {initial_count, opts} = Keyword.pop(opts, :initial_count, 0) args = [init_fun: fn -&gt; initial_count end, next_fun: &amp;(&amp;1 + 1)] Perpetual.start_link(args, opts) endand then you can use Counter, {Counter, name: :my_counter} or even {Counter, initial_count: 0, name: :my_counter} as a child specification.use Perpetual also accepts a list of options which configures the child specification and therefore how it runs under a supervisor. The generated child_spec/1 can be customized with the following options::id - the child specification identifier, defaults to the current module:restart - when the child should be restarted, defaults to :permanent:shutdown - how to shut down the child, either immediately or by giving it time to shut downFor example:use Perpetual, restart: :transient, shutdown: 10_000See the &quot;Child specification&quot; section in the Supervisor module for more detailed information. The @doc annotation immediately preceding use Perpetual will be attached to the generated child_spec/1 function.Name registrationA perpetual server is bound to the same name registration rules as GenServers. Read more about it in the GenServer documentation.A word on distributed perpetual serversIt is important to consider the limitations of distributed perpetual servers. Like Agents, Perpetual provides two APIs, one that works with anonymous functions and another that expects an explicit module, function, and arguments.In a distributed setup with multiple nodes, the API that accepts anonymous functions only works if the caller (client) and the server have the same version of the caller module.Keep in mind this issue also shows up when performing &quot;rolling upgrades&quot; with perpetual servers. By rolling upgrades we mean the following situation: you wish to deploy a new version of your software by shutting down some of your nodes and replacing them with nodes running a new version of the software. In this setup, part of your environment will have one version of a given module and the other part another version (the newer one) of the same module.The best solution is to simply use the explicit module, function, and arguments APIs when working with distributed perpetual servers.Hot code swappingA perpetual server can have its code hot swapped live by simply passing a module, function, and arguments tuple to the update instruction. For example, imagine you have a server named :sample and you want to convert its inner value from a keyword list to a map. It can be done with the following instruction:{:update, :sample, {:advanced, {Enum, :into, [%{}]}}}The server's current value will be added to the given list of arguments ([%{}]) as the first argument.","ref":"Perpetual.html","title":"Perpetual","type":"module"},{"doc":"Performs a cast (fire and forget) operation on the perpetual server's state.The function fun is sent to the perpetual which invokes the function passing the current state. The return value of fun becomes the new state of the server.Note that cast returns :ok immediately, regardless of whether perpetual (or the node it should live on) exists.Examplesiex&gt; {:ok, pid} = Perpetual.start_link(init_fun: fn -&gt; 0 end, next_fun: &amp;(&amp;1 + 1)) iex&gt; Perpetual.cast(pid, fn state -&gt; -1 * state end) :ok iex&gt; _value = Perpetual.get(pid, fn state -&gt; state end)","ref":"Perpetual.html#cast/2","title":"Perpetual.cast/2","type":"function"},{"doc":"Performs a cast (fire and forget) operation on the perpetual server's state.Same as cast/2 but a module, function, and arguments are expected instead of an anonymous function. The state is added as first argument to the given list of arguments.Examplesiex&gt; {:ok, pid} = Perpetual.start_link(init_fun: fn -&gt; 0 end, next_fun: &amp;(&amp;1 + 1)) iex&gt; Perpetual.cast(pid, Kernel, :*, [-1]) :ok iex&gt; _value = Perpetual.get(pid, fn state -&gt; state end)","ref":"Perpetual.html#cast/4","title":"Perpetual.cast/4","type":"function"},{"doc":"Returns a specification to start a perpetual server under a supervisor.See the &quot;Child specification&quot; section in the Supervisor module for more detailed information.","ref":"Perpetual.html#child_spec/1","title":"Perpetual.child_spec/1","type":"function"},{"doc":"Gets a perpetual server's value via the given anonymous function.The function fun is sent to the perpetual which invokes the function passing the server's state. The result of the function invocation is returned from this function.timeout is an integer greater than zero which specifies how many milliseconds are allowed before the server executes the function and returns the result value, or the atom :infinity to wait indefinitely. If no result is received within the specified time, the function call fails and the caller exits.Examplesiex&gt; {:ok, pid} = Perpetual.start_link(init_fun: fn -&gt; 0 end, next_fun: &amp;(&amp;1 + 1)) iex&gt; _value = Perpetual.get(pid, fn state -&gt; state end)","ref":"Perpetual.html#get/3","title":"Perpetual.get/3","type":"function"},{"doc":"Gets a perpetual server's value via the given function.Same as get/3 but a module, function, and arguments are expected instead of an anonymous function. The state is added as first argument to the given list of arguments.","ref":"Perpetual.html#get/5","title":"Perpetual.get/5","type":"function"},{"doc":"Gets and updates the perpetual server's state in one operation via the given anonymous function.The function fun is sent to the perpetual which invokes the function passing the current state. The function must return a tuple with two elements, the first being the value to return (that is, the &quot;get&quot; value) and the second one being the new state of the perpetual server.timeout is an integer greater than zero which specifies how many milliseconds are allowed before the server executes the function and returns the result value, or the atom :infinity to wait indefinitely. If no result is received within the specified time, the function call fails and the caller exits.Examplesiex&gt; {:ok, pid} = Perpetual.start_link(init_fun: fn -&gt; 0 end, next_fun: &amp;(&amp;1 + 1)) iex&gt; _current_value = Perpetual.get_and_update(pid, fn state -&gt; {state, -1 * state} end) iex&gt; _later_value = Perpetual.get(pid, fn state -&gt; state end)","ref":"Perpetual.html#get_and_update/3","title":"Perpetual.get_and_update/3","type":"function"},{"doc":"Gets and updates the perpetual state in one operation via the given function.Same as get_and_update/3 but a module, function, and arguments are expected instead of an anonymous function. The state is added as first argument to the given list of arguments.","ref":"Perpetual.html#get_and_update/5","title":"Perpetual.get_and_update/5","type":"function"},{"doc":"Starts a perpetual server process without links (outside of a supervision tree).See start_link/2 for more information.Examplesiex&gt; {:ok, pid} = Perpetual.start_link(init_fun: fn -&gt; 0 end, next_fun: &amp;(&amp;1 + 1)) iex&gt; _value = Perpetual.get(pid, fn state -&gt; state end)","ref":"Perpetual.html#start/2","title":"Perpetual.start/2","type":"function"},{"doc":"Starts a perpetual server linked to the current process with the given function.This is often used to start the server as part of a supervision tree.Once the server is spawned, the given function init_fun is invoked in the server process, and should return the initial server state. Note that start_link/2 does not return until the given function has returned.Once the server is running, the given function next_fun is invoked repeatedly in the server process in a loop. This function will be passed the current state and should return the next state.OptionsThe :name option is used for registration as described in the module documentation.If the :timeout option is present, the server is allowed to spend at most the given number of milliseconds on initialization or it will be terminated and the start function will return {:error, :timeout}.If the :debug option is present, the corresponding function in the :sys module will be invoked.If the :spawn_opt option is present, its value will be passed as options to the underlying process as in Process.spawn/4.Return valuesIf the server is successfully created and initialized, the function returns {:ok, pid}, where pid is the PID of the server. If an server with the specified name already exists, the function returns {:error, {:already_started, pid}} with the PID of that process.If the given function callback fails, the function returns {:error, reason}.Examplesiex&gt; {:ok, pid} = Perpetual.start_link(init_fun: fn -&gt; 0 end, next_fun: &amp;(&amp;1 + 1)) iex&gt; _value = Perpetual.get(pid, fn state -&gt; state end) iex&gt; {:error, {exception, _stacktrace}} = Perpetual.start(init_fun: fn -&gt; raise &quot;oops&quot; end, next_fun: &amp;(&amp;1 + 1)) iex&gt; exception %RuntimeError{message: &quot;oops&quot;}","ref":"Perpetual.html#start_link/2","title":"Perpetual.start_link/2","type":"function"},{"doc":"Synchronously stops the perpetual server with the given reason.It returns :ok if the server terminates with the given reason. If the server terminates with another reason, the call will exit.This function keeps OTP semantics regarding error reporting. If the reason is any other than :normal, :shutdown or {:shutdown, _}, an error report will be logged.Examplesiex&gt; {:ok, pid} = Perpetual.start_link(init_fun: fn -&gt; 0 end, next_fun: &amp;(&amp;1 + 1)) iex&gt; Perpetual.stop(pid) :ok","ref":"Perpetual.html#stop/3","title":"Perpetual.stop/3","type":"function"},{"doc":"Updates the perpetual server's state via the given anonymous function.The function fun is sent to the perpetual which invokes the function passing the current state. The return value of fun becomes the new state of the server.This function always returns :ok.timeout is an integer greater than zero which specifies how many milliseconds are allowed before the perpetual executes the function and returns the result value, or the atom :infinity to wait indefinitely. If no result is received within the specified time, the function call fails and the caller exits.Examplesiex&gt; {:ok, pid} = Perpetual.start_link(init_fun: fn -&gt; 0 end, next_fun: &amp;(&amp;1 + 1)) iex&gt; Perpetual.update(pid, fn state -&gt; -1 * state end) :ok iex&gt; _value = Perpetual.get(pid, fn state -&gt; state end)","ref":"Perpetual.html#update/3","title":"Perpetual.update/3","type":"function"},{"doc":"Updates the perpetual server's state via the given function.Same as update/3 but a module, function, and arguments are expected instead of an anonymous function. The state is added as first argument to the given list of arguments.Examplesiex&gt; {:ok, pid} = Perpetual.start_link(init_fun: fn -&gt; 0 end, next_fun: &amp;(&amp;1 + 1)) iex&gt; Perpetual.update(pid, Kernel, :*, [-1]) :ok iex&gt; _value = Perpetual.get(pid, fn state -&gt; state end)","ref":"Perpetual.html#update/5","title":"Perpetual.update/5","type":"function"},{"doc":"The perpetual server's initial state function","ref":"Perpetual.html#t:init_fun_or_mfa/0","title":"Perpetual.init_fun_or_mfa/0","type":"type"},{"doc":"The perpetual server name","ref":"Perpetual.html#t:name/0","title":"Perpetual.name/0","type":"type"},{"doc":"The perpetual server's next state function","ref":"Perpetual.html#t:next_fun_or_mfa/0","title":"Perpetual.next_fun_or_mfa/0","type":"type"},{"doc":"Return values of start* functions","ref":"Perpetual.html#t:on_start/0","title":"Perpetual.on_start/0","type":"type"},{"doc":"The perpetual server reference","ref":"Perpetual.html#t:perpetual/0","title":"Perpetual.perpetual/0","type":"type"},{"doc":"The perpetual server value","ref":"Perpetual.html#t:state/0","title":"Perpetual.state/0","type":"type"},{"doc":"PerpetualPerpetual is a simple abstraction around repeatedly iterating state in Elixir.It is similar to Elixir's Agent module in that it can share or store state that must be accessed from different processes or by the same process at different points in time, and in addition to that, Perpetual lets you define a function for repeatedly updating the stored state for as long as the process is kept running.","ref":"readme.html","title":"Perpetual","type":"extras"},{"doc":"This is an early work in progress, and should be considered experimental, incomplete, and unstable until v1.0.0, following Semantic Versioning.All notable changes will be recorded in the changelog.","ref":"readme.html#status","title":"Perpetual - Status","type":"extras"},{"doc":"Once available in Hex, the package can be installed by adding perpetual to your list of dependencies in mix.exs. For now, you can test the development branch with:def deps do [ {:perpetual, github: &quot;xtagon/perpetual&quot;, branch: &quot;edge&quot;} ] end","ref":"readme.html#installation","title":"Perpetual - Installation","type":"extras"},{"doc":"Documentation can be generated from the source code using mix docs.","ref":"readme.html#documentation","title":"Perpetual - Documentation","type":"extras"},{"doc":"The following Mix tasks are available to assist in development:mix docsmix testmix coverallsmix credo","ref":"readme.html#development","title":"Perpetual - Development","type":"extras"},{"doc":"Perpetual's source code is released under Apache License 2.0.Check NOTICE and LICENSE files for more information.","ref":"readme.html#license","title":"Perpetual - License","type":"extras"},{"doc":"ChangelogAll notable changes to this project will be documented in this file.The format is based on Keep a Changelog, and this project adheres to Semantic Versioning.","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"","ref":"changelog.html#unreleased","title":"Changelog - Unreleased","type":"extras"},{"doc":"Apache License Version 2.0, January 2004 http://www.apache.org/licenses/ TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION 1. Definitions. &quot;License&quot; shall mean the terms and conditions for use, reproduction, and distribution as defined by Sections 1 through 9 of this document. &quot;Licensor&quot; shall mean the copyright owner or entity authorized by the copyright owner that is granting the License. &quot;Legal Entity&quot; shall mean the union of the acting entity and all other entities that control, are controlled by, or are under common control with that entity. For the purposes of this definition, &quot;control&quot; means (i) the power, direct or indirect, to cause the direction or management of such entity, whether by contract or otherwise, or (ii) ownership of fifty percent (50%) or more of the outstanding shares, or (iii) beneficial ownership of such entity. &quot;You&quot; (or &quot;Your&quot;) shall mean an individual or Legal Entity exercising permissions granted by this License. &quot;Source&quot; form shall mean the preferred form for making modifications, including but not limited to software source code, documentation source, and configuration files. &quot;Object&quot; form shall mean any form resulting from mechanical transformation or translation of a Source form, including but not limited to compiled object code, generated documentation, and conversions to other media types. &quot;Work&quot; shall mean the work of authorship, whether in Source or Object form, made available under the License, as indicated by a copyright notice that is included in or attached to the work (an example is provided in the Appendix below). &quot;Derivative Works&quot; shall mean any work, whether in Source or Object form, that is based on (or derived from) the Work and for which the editorial revisions, annotations, elaborations, or other modifications represent, as a whole, an original work of authorship. For the purposes of this License, Derivative Works shall not include works that remain separable from, or merely link (or bind by name) to the interfaces of, the Work and Derivative Works thereof. &quot;Contribution&quot; shall mean any work of authorship, including the original version of the Work and any modifications or additions to that Work or Derivative Works thereof, that is intentionally submitted to Licensor for inclusion in the Work by the copyright owner or by an individual or Legal Entity authorized to submit on behalf of the copyright owner. For the purposes of this definition, &quot;submitted&quot; means any form of electronic, verbal, or written communication sent to the Licensor or its representatives, including but not limited to communication on electronic mailing lists, source code control systems, and issue tracking systems that are managed by, or on behalf of, the Licensor for the purpose of discussing and improving the Work, but excluding communication that is conspicuously marked or otherwise designated in writing by the copyright owner as &quot;Not a Contribution.&quot; &quot;Contributor&quot; shall mean Licensor and any individual or Legal Entity on behalf of whom a Contribution has been received by Licensor and subsequently incorporated within the Work. 2. Grant of Copyright License. Subject to the terms and conditions of this License, each Contributor hereby grants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free, irrevocable copyright license to reproduce, prepare Derivative Works of, publicly display, publicly perform, sublicense, and distribute the Work and such Derivative Works in Source or Object form. 3. Grant of Patent License. Subject to the terms and conditions of this License, each Contributor hereby grants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free, irrevocable (except as stated in this section) patent license to make, have made, use, offer to sell, sell, import, and otherwise transfer the Work, where such license applies only to those patent claims licensable by such Contributor that are necessarily infringed by their Contribution(s) alone or by combination of their Contribution(s) with the Work to which such Contribution(s) was submitted. If You institute patent litigation against any entity (including a cross-claim or counterclaim in a lawsuit) alleging that the Work or a Contribution incorporated within the Work constitutes direct or contributory patent infringement, then any patent licenses granted to You under this License for that Work shall terminate as of the date such litigation is filed. 4. Redistribution. You may reproduce and distribute copies of the Work or Derivative Works thereof in any medium, with or without modifications, and in Source or Object form, provided that You meet the following conditions: (a) You must give any other recipients of the Work or Derivative Works a copy of this License; and (b) You must cause any modified files to carry prominent notices stating that You changed the files; and (c) You must retain, in the Source form of any Derivative Works that You distribute, all copyright, patent, trademark, and attribution notices from the Source form of the Work, excluding those notices that do not pertain to any part of the Derivative Works; and (d) If the Work includes a &quot;NOTICE&quot; text file as part of its distribution, then any Derivative Works that You distribute must include a readable copy of the attribution notices contained within such NOTICE file, excluding those notices that do not pertain to any part of the Derivative Works, in at least one of the following places: within a NOTICE text file distributed as part of the Derivative Works; within the Source form or documentation, if provided along with the Derivative Works; or, within a display generated by the Derivative Works, if and wherever such third-party notices normally appear. The contents of the NOTICE file are for informational purposes only and do not modify the License. You may add Your own attribution notices within Derivative Works that You distribute, alongside or as an addendum to the NOTICE text from the Work, provided that such additional attribution notices cannot be construed as modifying the License. You may add Your own copyright statement to Your modifications and may provide additional or different license terms and conditions for use, reproduction, or distribution of Your modifications, or for any such Derivative Works as a whole, provided Your use, reproduction, and distribution of the Work otherwise complies with the conditions stated in this License. 5. Submission of Contributions. Unless You explicitly state otherwise, any Contribution intentionally submitted for inclusion in the Work by You to the Licensor shall be under the terms and conditions of this License, without any additional terms or conditions. Notwithstanding the above, nothing herein shall supersede or modify the terms of any separate license agreement you may have executed with Licensor regarding such Contributions. 6. Trademarks. This License does not grant permission to use the trade names, trademarks, service marks, or product names of the Licensor, except as required for reasonable and customary use in describing the origin of the Work and reproducing the content of the NOTICE file. 7. Disclaimer of Warranty. Unless required by applicable law or agreed to in writing, Licensor provides the Work (and each Contributor provides its Contributions) on an &quot;AS IS&quot; BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied, including, without limitation, any warranties or conditions of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A PARTICULAR PURPOSE. You are solely responsible for determining the appropriateness of using or redistributing the Work and assume any risks associated with Your exercise of permissions under this License. 8. Limitation of Liability. In no event and under no legal theory, whether in tort (including negligence), contract, or otherwise, unless required by applicable law (such as deliberate and grossly negligent acts) or agreed to in writing, shall any Contributor be liable to You for damages, including any direct, indirect, special, incidental, or consequential damages of any character arising as a result of this License or out of the use or inability to use the Work (including but not limited to damages for loss of goodwill, work stoppage, computer failure or malfunction, or any and all other commercial damages or losses), even if such Contributor has been advised of the possibility of such damages. 9. Accepting Warranty or Additional Liability. While redistributing the Work or Derivative Works thereof, You may choose to offer, and charge a fee for, acceptance of support, warranty, indemnity, or other liability obligations and/or rights consistent with this License. However, in accepting such obligations, You may act only on Your own behalf and on Your sole responsibility, not on behalf of any other Contributor, and only if You agree to indemnify, defend, and hold each Contributor harmless for any liability incurred by, or claims asserted against, such Contributor by reason of your accepting any such warranty or additional liability. END OF TERMS AND CONDITIONS","ref":"license.html","title":"LICENSE","type":"extras"},{"doc":"LEGAL NOTICE INFORMATION ------------------------ All the files in this distribution are copyright to the terms below. == lib/perpetual.ex == lib/perpetual/server.ex == test/perpetual_test.exs These files are derivative works based on the following files from Elixir's source code: - lib/elixir/lib/agent.ex - lib/elixir/lib/agent/server.ex - lib/elixir/test/elixir/agent_test.exs The original source is available here: https://github.com/elixir-lang/elixir/tree/v1.11.1/lib/elixir/lib Notable changes: - Renamed `Agent` to `Perpetual` (new namespace, not to conflict with Agent) - Extended the original implementation beyond Agent's original functionality (for example, specifying a &quot;next&quot; function instead of just an initial function, and using GenServer timeouts to iterate automatically) - Extended the original tests to fit this new implementation's behavior - Modified the documentation slightly to fit this new implementation's API, although most of the documentation is the same as the original documentation wherever the behavior stayed the same. The copyright and license for the original sources upon which this derivative work is based are: Copyright 2012 Plataformatec Licensed under the Apache License, Version 2.0 (the &quot;License&quot;); you may not use this file except in compliance with the License. You may obtain a copy of the License at https://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an &quot;AS IS&quot; BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License. == All other files and significant changes Copyright 2020 Justin Workman Licensed under the Apache License, Version 2.0 (the &quot;License&quot;); you may not use this file except in compliance with the License. You may obtain a copy of the License at https://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an &quot;AS IS&quot; BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License.","ref":"notice.html","title":"NOTICE","type":"extras"}]